//--  Copyright (c) 2003 VoiceXML Forum.  All rights reserved. -->
//--  -- VOICEXML FORUM CONFIDENTIAL ------------------------  -->
//--  PRELIMINARY VERSION FOR DISTRIBUTION TO MEMBERS ONLY.--  -->

package org.voicexml.conformance.testharness;

/**
 * AssertionTestLoader reads assertion test meta data from the manifest.xml
 * file.  One instance will be created upon application initiation and
 * shared by all active user sessions.
 *
 * @author Jonathan Engelsma
 * @author Jeff Haynie
 */
import java.io.*;
import java.util.*;

// import org.apache.xerces.dom.TextImpl;
import org.apache.xerces.parsers.DOMParser;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.SAXException;
// import org.xml.sax.SAXNotRecognizedException;
// import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;

public class AssertionTestLoader implements org.xml.sax.ErrorHandler
{
    private Hashtable assertionMap = null;

    private String baseDir = null;

    private String coreTestSuite = null;

    private static AssertionTestLoader instance = null;

    /**
     * Returns a reference to the AssertionTestLoader singleton.
     */
    public static synchronized AssertionTestLoader getInstance(String WebAppDirectory, String DocumentRoot, String TestSuite)
    {
        instance =  new AssertionTestLoader(WebAppDirectory, DocumentRoot, TestSuite);

        return instance;
    }

    private boolean isSameTestSuite(String TestSuite)
    {
	if (this.coreTestSuite == TestSuite) return true;
	return false;
    }

    /**
     * AssertTestLoader constructor.
     */
    private AssertionTestLoader(String WebAppDirectory, String DocumentRoot, String TestSuite)
    {
        System.err.println("AssertionTestLoader(" + WebAppDirectory + ", " + DocumentRoot + ", " + TestSuite + ")");
	if (this.coreTestSuite != TestSuite) {
		this.coreTestSuite = TestSuite;

        	// Store test meta data, hashed by assertion id.
        	this.assertionMap = new Hashtable();

        	// create a parser and register the error handler.
        	DOMParser parser = new DOMParser();
        	parser.setErrorHandler(this);
	
        	PrintWriter out = new PrintWriter(System.out);

        	InputStream in = getClass().getClassLoader().getResourceAsStream(TestSuite + "/" + "manifest.xml");
        	if ( in == null )
        	{
			String manifestUrl = null;

			if (DocumentRoot != null) {
				manifestUrl = DocumentRoot + "/" + TestSuite +"/manifest.xml";
			}
			
			if (WebAppDirectory != null) {
				this.baseDir = WebAppDirectory ;
				// this.baseDir = WebAppDirectory + "/" + this.baseDir;
			}

			if ((manifestUrl == null) || (this.baseDir == null)) {
				try {
					ResourceBundle rb = ResourceBundle.getBundle("config");
					if (manifestUrl == null) { 
						manifestUrl = rb.getString("org.voicexml.conformance.testharness.rootdirectory"); 
						if (manifestUrl != null) manifestUrl = manifestUrl + "/" + TestSuite;
					}
					if (this.baseDir == null) { 
						this.baseDir = rb.getString("org.voicexml.conformance.testharness.basedir"); 
						if (this.baseDir != null) this.baseDir = this.baseDir + "/" + TestSuite;
					}
				}
				catch (Exception e) {
					e.printStackTrace();
					if (manifestUrl == null) { manifestUrl = "http://localhost:1080/ir/manifest.xml"; }
					if (this.baseDir == null) { this.baseDir = "C:\\Program Files\\Apache Group\\Tomcat 4.1\\webapps\\"; }
				}
			}

			out.println("manifest @ " + manifestUrl);
			out.println("baseDir @ " + this.baseDir);

			try {
			parser.parse(manifestUrl);
			} catch (Exception e) {
				out.println("Failed to find Manifest");
				e.printStackTrace(out);
			}
        	}
        	else
        	{
			out.println("Found Manifest inside WAR file");

			// parse from inside the war first if possible
			try {
				parser.parse(new InputSource(in));
			} catch (Exception e) {
				e.printStackTrace(out);
			}
		}
        	Document doc = parser.getDocument();

        	if(doc != null) {
            		loadTestCases(out,doc,"test");
        	} else {
            		System.err.println("Failed to load manifest.xml");
        	}
	} else {
                System.err.println("Not reloading Manifest because we are using the same testSuite");
	}

    }

    /**
     * Returns the basedir property setting.
     */
    public String getBaseDir()
    {
        return this.baseDir;
    }

    public void writeVXMLOutput (Writer writer, String gramType, String testId, String testSuite)
      throws Exception
    {
		String location = testSuite + File.separatorChar + gramType + File.separatorChar + getStartUri(testId);
		File f = new File(baseDir,location);
        	PrintWriter out = new PrintWriter(System.out);
		out.println("VXML @ " + location);
		InputStream in = null;
		if (f.exists())
		{
			in = new FileInputStream(f);
		}
		else
		{
			String path = location.replace(File.separatorChar,'/');
			System.out.println("looking for "+ path);
			// look in the WAR file itself
			in = getClass().getClassLoader().getResourceAsStream(path);
		}

		PrintWriter w = new PrintWriter(writer);

		if (false == writeOutput(w,in))
		{
			w.println("Couldn't find test source for: "+testId);
		}
	}
    public void writeTXMLOutput (Writer writer, String gramType, String testId, String testSuite)
      throws Exception
    {
		String location = testSuite + File.separatorChar + "txml" + File.separatorChar + getStartUri(testId);
        	PrintWriter out = new PrintWriter(System.out);
		out.println("TXML @ " + location);

		int idx = location.indexOf(".vxml");

		if(idx != -1) {

			location = location.substring(0,idx) + ".txml";

		}
		File f = new File(baseDir,location);
		InputStream in = null;
		if (f.exists())
		{
			in = new FileInputStream(f);
		}
		else
		{
			String path = location.replace(File.separatorChar,'/');
			System.out.println("looking for "+ path);
			// look in the WAR file itself
			in = getClass().getClassLoader().getResourceAsStream(path);
		}

		PrintWriter w = new PrintWriter(writer);

		if (false == writeOutput(w,in))
		{
			w.println("Couldn't find test source for: "+testId);
		}
	}
	public void writeDependencyOutput (Writer writer, String gramType, String uri, String testSuite)
      throws Exception
    {
		int idx = uri.indexOf(".vxml");

		if(idx != -1) {
			uri = uri.substring(0,idx) + ".txml";
		}
		String location = testSuite + File.separatorChar + "txml" + File.separatorChar + uri;
		System.out.println("(dep) TXML @ " + baseDir +location);
		File f = new File(baseDir,location);
		InputStream in = null;
		if (f.exists())
		{
			in = new FileInputStream(f);
		}
		else
		{
			String path = location.replace(File.separatorChar,'/');
			System.out.println("looking for "+ path);
			// look in the WAR file itself
			in = getClass().getClassLoader().getResourceAsStream(path);
		}

		PrintWriter w = new PrintWriter(writer);

		if (false == writeOutput(w,in))
		{
			w.println("Couldn't find test source for: "+uri);
			w.println("Couldn't find test source for: "+location);
		}
	}
	public boolean writeOutput (PrintWriter writer, InputStream in)
 	 	throws Exception
	{
		if (null != in)
		{
			byte buf[]=new byte[4096];
			while ( true )
			{
				int c = in.read(buf);
				if (c<0)
				{
					break;
				}
				String outStr = new String(buf,0,c);
            	                writer.print(validHTML(outStr));
			}
			in.close();
		}
		else
		{
			return false;
		}
		writer.flush();
		return true;
	}

    /**
     * Given an assertion id, find out what section in the spec that assertion
     * was drawn from.
     * @param assertId The assertion id.
     * @return The section the assertion was drawn from.
     */
    public String getSection(String assertId)
    {
        if(assertId == null) {
            return null;
        }
        Dictionary assertInfo = (Dictionary) this.assertionMap.get(assertId);
        String section = null;
        if(assertInfo != null) {
            Node assertNode = (Node)assertInfo.get("assert");
            if(assertNode != null) {
                NamedNodeMap attrs = assertNode.getAttributes();
                if(attrs != null) {
                    Attr specAttr = (Attr) attrs.getNamedItem("spec");
                    if(specAttr != null) {
                        section = specAttr.getValue();
                    }
                }
            }
        }
        return section;
    }

    /**
     * Given an assertion id, lookup the text for that assertion.
     * @param assertId The assertion id.
     * @return The text for that assertion.
     */
    public String getAssertionText(String assertId)
    {
        if(assertId == null) {
            return null;
        }
        Dictionary assertInfo = (Dictionary) this.assertionMap.get(assertId);
        String assertText = "";
        if(assertInfo != null) {
            Node assertNode = (Node)assertInfo.get("assert");
            if(assertNode != null) {
                NodeList assertNodes = assertNode.getChildNodes();
                for(int i=0; i<assertNodes.getLength(); i++) {
                    Node childNode = assertNodes.item(i);
                    String nodeName = childNode.getNodeName();
                    if(nodeName.equals("#text") || nodeName.equals("#cdata-section")) {
                        assertText += childNode.getNodeValue();
                        //break;
                    } else {
                        //  tags are embedded in the assertions.  :-(
                        assertText += "<" + nodeName + ">";
                        NodeList tagNodes = childNode.getChildNodes();
                        for(int j=0; j<tagNodes.getLength(); j++) {
                            Node tagNode = tagNodes.item(j);
                            if(tagNode.getNodeName().equals("#text")) {
                                assertText += tagNode.getNodeValue();
                            }
                        }
                        assertText += "</" + nodeName + ">";
                    }
                }
            }
        }
        return assertText;
    }

    /**
     * Determine the relative start URI of a given assertion id.
     * @param assertId The assertion id.
     * @return The relative starting URI for the given assertion, or a null
     * if an non-existing assertion id is given.
     */
    public String getStartUri(String assertId)
    {
        if(assertId == null) {
            System.err.println("getStartUri - no assertId");
            return null;
        }

        Dictionary assertInfo = (Dictionary) this.assertionMap.get(assertId);
        String startUri = null;
        if(assertInfo != null) {
            Node startElement = (Node) assertInfo.get("start");
            if(startElement != null) {
                NamedNodeMap attrs = startElement.getAttributes();
                if(attrs != null) {
                    Attr uriAttr = (Attr) attrs.getNamedItem("uri");
                    if(uriAttr != null) {
                        startUri = uriAttr.getValue();
                    } else {
            		System.err.println("getStartUri - failed to get start node uri attribute");
		    }
                } else {
            	    System.err.println("getStartUri - failed to get start node attributes");
		}
            } else {
            	System.err.println("getStartUri - failed to get start node");
	    } 
        } else {
            System.err.println("getStartUri - failed to get dictionary");
	}

        // transforms are done a prior, so the file extension has to be massaged.
        if(startUri != null) {
            int idx = startUri.indexOf(".txml");
            if(idx != -1) {
                startUri = startUri.substring(0,idx) + ".vxml";
            }
        }

        return startUri;
    }

    /**
     * Determine the dependency files for a given assertion id.
     * @param assertId The assertion id.
     * @return A Dictionary with keys set to the dependency URIs, and values set
     * to the dependency's mime-type.
     */
    public Dictionary getTestDependencies(String assertId)
    {
        if(assertId == null) {
            return null;
        }

        Hashtable retval = null;

        Dictionary assertInfo = (Dictionary) this.assertionMap.get(assertId);

        if(assertInfo != null) {
            Vector deps = (Vector) assertInfo.get("deps");
            if((deps != null) && (deps.size() > 0))  {
                retval = new Hashtable();
                for(int i=0; i<deps.size(); i++) {
                    String depUri = null;
                    String depType = null;
                    Node depElement = (Node) deps.elementAt(i);
                    if(depElement != null) {
                        NamedNodeMap attrs = depElement.getAttributes();
                        if(attrs != null) {
                            Attr uriAttr = (Attr) attrs.getNamedItem("uri");
                            if(uriAttr != null) {
                                depUri = uriAttr.getValue();
                                // txml transforms are done a prior, so the file extension has to be massaged.
                                if(depUri != null) {
                                    int idx = depUri.indexOf(".txml");
                                    if(idx != -1) {
                                        depUri = depUri.substring(0,idx) + ".vxml";
                                    }
                                }
                            }
                            Attr typeAttr = (Attr) attrs.getNamedItem("type");
                            if(typeAttr != null) {
                                depType = typeAttr.getValue();
                            }
                            if((depUri != null) && (depType != null)) {
                                retval.put(depUri, depType);
                            }
                        }
                    }
                }
            }
        }
        return retval;
    }


    /**
     * Traverse the DOM tree picking out the test elements.
     */
    protected void loadTestCases(PrintWriter out, Document document,
                             String elementName) {

        // get elements that match
        NodeList elements = document.getElementsByTagName(elementName);

        // is there anything to do?
        if (elements == null) {
            return;
        }


        int elementCount = elements.getLength();
        for (int i = 0; i < elementCount; i++) {
            Element element = (Element)elements.item(i);
            addTestElementToMap(out, element);
        }

    }

    /**
     * Adds a test element to the assertion map.
     */
    protected void addTestElementToMap(PrintWriter out, Element element)
    {

        NodeList childElements = element.getChildNodes();
        Vector deps = null;
        String assertId = null;
        Hashtable assertionData = new Hashtable();

        // Process the children of the test case.
        for(int i=0; i<childElements.getLength(); i++) {
            Node child = childElements.item(i);
            if(child != null) {
                String childName = child.getNodeName().intern();
                if(childName == "start") {
                    assertionData.put("start",child);
                } else if(childName == "assert") {
                    assertionData.put("assert",child);
                    NamedNodeMap attrs = child.getAttributes();
                    Attr idAttr = (Attr)attrs.getNamedItem("id");
                    if(idAttr != null) {
                        assertId = idAttr.getNodeValue();
                    }
                } else if(childName == "dep") {
                    if(deps==null) {
                        deps = new Vector();
                    }
                    deps.add(child);
                }
            }
        }

        /**
         * One of the children nodes has to be an <assert> element.
         */
        if(assertId != null) {
            out.println("Adding assertion " + assertId);
            out.flush();
            if(deps != null) {
                assertionData.put("deps",deps);
            }
            this.assertionMap.put(assertId,assertionData);
        }
    }

    /**
     * Normalizes the given string.
     */
    protected static String normalize(String s) {
        StringBuffer str = new StringBuffer();

        int len = (s != null) ? s.length() : 0;
        for (int i = 0; i < len; i++) {
            char ch = s.charAt(i);
            switch (ch) {
            case '<': {
                    str.append("&lt;");
                    break;
                }
            case '>': {
                    str.append("&gt;");
                    break;
                }
            case '&': {
                    str.append("&amp;");
                    break;
                }
            case '"': {
                    str.append("&quot;");
                    break;
                }
            case '\r':
            case '\n': {
                    str.append("&#");
                    str.append(Integer.toString(ch));
                    str.append(';');
                    break;
                }
            default: {
                    str.append(ch);
                }
            }
        }
        return str.toString();
    }

    //
    // ErrorHandler methods
    //

    /** Warning. */
    public void warning(SAXParseException ex) throws SAXException {
        printError("Warning", ex);
    }

    /** Error. */
    public void error(SAXParseException ex) throws SAXException {
        printError("Error", ex);
    }

    /** Fatal error. */
    public void fatalError(SAXParseException ex) throws SAXException {
        printError("Fatal Error", ex);
        throw ex;
    }

    /**
     * Print out parse error msgs to standard err.
     */
    protected void printError(String type, SAXParseException ex) {

        System.err.print("[");
        System.err.print(type);
        System.err.print("] ");
        String systemId = ex.getSystemId();
        if (systemId != null) {
            int index = systemId.lastIndexOf('/');
            if (index != -1)
                systemId = systemId.substring(index + 1);
            System.err.print(systemId);
        }
        System.err.print(':');
        System.err.print(ex.getLineNumber());
        System.err.print(':');
        System.err.print(ex.getColumnNumber());
        System.err.print(": ");
        System.err.print(ex.getMessage());
        System.err.println();
        System.err.flush();
    }
	public static String validHTML (String original)
	{
		StringBuffer ny = new StringBuffer(original.length());
		char next;
		boolean special;

		System.err.println("validHTML() - " + original);


		for (int i = 0; i<original.length(); i++)
		{
			next = original.charAt(i);
			special = false;

			if (next=='>')
			{
				special=true;
				ny.append("&gt;");
			}
			if (next=='<')
			{
				special=true;
				ny.append("&lt;");
			}
			if (next=='&')
			{
				special=true;
				ny.append("&amp;");
			}
			if (next=='"')
			{
				special=true;
				ny.append("&quot;");
			}
	/*

			if (next=='\'')
			{
				special=true;
				ny.append("&$#39;");
			}
	*/

			if (!special)
			{
				ny.append(next);
			}
		}

		return ny.toString();
	}


    /**
     * Tests.
     */
    public static void main(String[] args) {
        String dummy = null;
        AssertionTestLoader atl = AssertionTestLoader.getInstance(dummy, dummy, dummy);
        System.out.println("assertion null=" + atl.getStartUri(null));
        System.out.println("assertion 1=" + atl.getStartUri("1"));
        System.out.println("assertion 1040=" + atl.getStartUri("1"));
        System.out.println("assertion 1185=" + atl.getStartUri("1185"));
        System.out.println("assertion bad=" + atl.getStartUri("bad"));
        System.out.println("assertdesc 1=" + atl.getAssertionText("1"));
        System.out.println("assertspec 1=" + atl.getSection("1"));
        System.out.println("test complete.");
    }
}
