<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Base_test_href may be modified by vendors to point to the test location. -->
<xsl:param name="base_test_href"></xsl:param>
<xsl:param name="css">http://www.w3.org/StyleSheets/base.css</xsl:param>
<xsl:param name="spec_href">http://www.w3.org/TR/voicexml20/</xsl:param>


<!-- Extracted from the vendor's xml file. -->
<xsl:variable name="vendor" select="document('vendor_conformance.xml')/vendor"/>
<xsl:variable name="implementation" select="$vendor/implementation"/>
<xsl:variable name="supplier" select="$vendor/supplier"/>
<xsl:variable name="omissions" select="$vendor/omissions"/>
<xsl:variable name="notes" select="$vendor/notes"/>


<xsl:template match="tests">
  <html xml:lang="en" lang="en" xmlns="http://www.w3.org/1999/xhtml">
    <head>
      <title>VoiceXML 2.0 Certification Suite - ICS</title>
      <style type="text/css" xml:space="preserve"><![CDATA[
        table.testlist { border: black; border-width: 1px; }
        table.testlist th { background-color: rgb(180,255,180); font-size: 9pt; }
        table.testlist td { background-color: rgb(234,255,234); font-size: 9pt; } ]]>
      </style>
      <link rel="stylesheet" type="text/css" href="{$css}"/>
    </head>

    <body>

      <h1>VoiceXML 2.0 Certification Suite - Version 1.0</h1>
      <h3>Implementation Conformance Statement</h3>
      <br/><br/><br/><br/>

      <table class="testlist" cellspacing="2px" cellpadding="2px">
      <caption align="left"><h3>Implementation under test</h3></caption>
      <tr>
        <th>Question</th>
        <th width="650">Response</th>
      </tr>
      <tr><td width="150">Implementation Name:</td><td><xsl:value-of select="$implementation/name"/></td></tr>
      <tr><td width="150">Implementation Identifier:</td><td><xsl:value-of select="$implementation/version"/></td></tr>
      <tr><td width="150">OS Version:</td><td><xsl:value-of select="$implementation/os"/></td></tr>
      <tr><td width="150">Telephony Config:</td><td><xsl:value-of select="$implementation/telephony"/></td></tr>
      </table>

      <br/><br/><br/><br/>


      <table class="testlist" cellspacing="2px" cellpadding="2px">
      <caption align="left"><h3>Supplier information</h3></caption>
      <tr>
        <th>Question</th>
        <th width="650">Response</th>
      </tr>
      <tr><td width="150">Organization Name:</td><td><xsl:value-of select="$supplier/name"/></td></tr>
      <tr><td width="150">Organization Address:</td><td><xsl:value-of select="$supplier/address"/></td></tr>
      <tr><td width="150">Contact Name:</td><td><xsl:value-of select="$supplier/contact/name"/></td></tr>
      <tr><td width="150">Contact Email:</td><td><xsl:value-of select="$supplier/contact/email"/></td></tr>
      <tr><td width="150">Contact Phone:</td><td><xsl:value-of select="$supplier/contact/phone"/></td></tr>
      <tr><td width="150">Contact Fax:</td><td><xsl:value-of select="$supplier/contact/fax"/></td></tr>
      </table>

      <br/><br/><br/><br/>


      <table class="testlist" cellspacing="2px" cellpadding="2px">
      <caption align="left"><h3>Optional assertions</h3></caption>
      <tr>
        <th>Test Ommited?</th>
        <xsl:call-template name="table_macro"/>
      </tr>

      <xsl:for-each select="test/assert">
        <xsl:sort data-type="number" select="@id"/>
        <xsl:if test="@conf_level='optional' and not(@tsd='1') and not(@untestable='1')">
          <xsl:variable name="outer_id" select="@id"/>
          <xsl:variable name="selected" select="false"/>
          <tr>
            <td valign="top">
              <xsl:for-each select="$omissions/assert">
                <xsl:if test="@id=$outer_id">Yes</xsl:if>
              </xsl:for-each>
            </td>
            <xsl:call-template name="assert_macro"/>
          </tr>
        </xsl:if>
      </xsl:for-each>
      </table>

      <br/><br/><br/><br/>

      <table class="testlist" cellspacing="2px" cellpadding="2px">
      <caption align="left"><h3>Required assertions</h3></caption>
      <tr>
        <xsl:call-template name="table_macro"/>
      </tr>

      <xsl:for-each select="test/assert">
        <xsl:sort data-type="number" select="@id"/>
        <xsl:if test="not(@conf_level='optional') and not(@tsd='1') and not(@untestable='1')">
          <tr><xsl:call-template name="assert_macro"/></tr>
        </xsl:if>
      </xsl:for-each>
      </table>

      <br/><br/><br/><br/>

      <table class="testlist" cellspacing="2px" cellpadding="2px">
      <caption align="left"><h3>Assertions in dispute</h3> Note: These assertions are presented for informational purposes only.  No statement of conformance can be derived because there is no right or wrong behavior defined.</caption>
      <tr>
        <xsl:call-template name="table_macro"/>
      </tr>

      <xsl:for-each select="test/assert">
        <xsl:sort data-type="number" select="@id"/>
        <xsl:if test="@tsd='1'">
          <tr><xsl:call-template name="assert_macro"/></tr>
        </xsl:if>
      </xsl:for-each>
      </table>

      <br/><br/><br/><br/>

      <table class="testlist" cellspacing="2px" cellpadding="2px">
      <caption align="left"><h3>Untestable assertions</h3> Note: These assertions are presently untestable within CT Labs.  When the assertions become testable, they will be joined back into the main test set.</caption>
      <tr>
        <xsl:call-template name="table_macro"/>
      </tr>

      <xsl:for-each select="test/assert">
        <xsl:sort data-type="number" select="@id"/>
        <xsl:if test="@untestable='1'">
          <tr><xsl:call-template name="assert_macro"/></tr>
        </xsl:if>
      </xsl:for-each>
      </table>

    </body>
</html>
</xsl:template>


<!-- Fill in the column values that are constant across all sections. -->
<xsl:template name="assert_macro">
  <td valign="top"><a href="{$base_test_href}{../start/@uri}"><xsl:value-of select="@id"/></a></td>
  <td valign="top"><a target="spec" href="{$spec_href}#dml{@spec}">[<xsl:value-of select="@spec"/>]</a></td>
  <td valign="top"><xsl:choose><xsl:when test="@telephony_dependant='1'">Yes</xsl:when><xsl:otherwise>-</xsl:otherwise></xsl:choose></td>
  <td valign="top"><xsl:choose><xsl:when test="@exec_manual='1'">Yes</xsl:when><xsl:otherwise>-</xsl:otherwise></xsl:choose></td>
  <td valign="top"><xsl:choose><xsl:when test="@abs_uri='1'">Yes</xsl:when><xsl:otherwise>-</xsl:otherwise></xsl:choose></td>
  <td valign="top"><xsl:choose><xsl:when test="@disconnects='1'">Yes</xsl:when><xsl:otherwise>-</xsl:otherwise></xsl:choose></td>
  <td valign="top"><xsl:apply-templates/></td>
  <td valign="top">
    <xsl:variable name="outer_id" select="@id"/>
    <xsl:for-each select="$notes/assert">
      <xsl:if test="@id=$outer_id"><xsl:apply-templates/></xsl:if>
    </xsl:for-each>
  </td>
</xsl:template>

<!-- Declare collumn that are constant across all sections. -->
<xsl:template name="table_macro">
  <th>ID</th>
  <th>Spec. Reference</th>
  <th>Telephony Dependant</th>
  <th>Manual Examination</th>
  <th>URI Modification</th>
  <th>Auto Disconnect</th>
  <th>Abstract</th>
  <th>Vendor Notes</th>
</xsl:template>


</xsl:stylesheet>
